@@ -1,7 +1,7 @@
 /*
  * ct_table.h
  *
- * Copyright 2017-2019 Giuseppe Penone <giuspen@gmail.com>
+ * Copyright 2017-2020 Giuseppe Penone <giuspen@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,8 @@
 class CtTableCell : public CtTextCell, public Gtk::Bin
 {
 public:
-    CtTableCell(const Glib::ustring& textContent,
+    CtTableCell(CtMainWin* pCtMainWin,
+                const Glib::ustring& textContent,
                 const Glib::ustring& syntaxHighlighting);
     virtual ~CtTableCell();
 };
@@ -38,7 +39,8 @@ typedef std::list<CtTableRow> CtTableMatrix;
 class CtTable : public CtAnchoredWidget
 {
 public:
-    CtTable(const CtTableMatrix& tableMatrix,
+    CtTable(CtMainWin* pCtMainWin,
+            const CtTableMatrix& tableMatrix,
             const int colMin,
             const int colMax,
             const bool headFront,
@@ -46,12 +48,13 @@ public:
             const std::string& justification);
     virtual ~CtTable();
 
-    virtual void applyWidthHeight(const int /*parentTextWidth*/) override {}
-    virtual void to_xml(xmlpp::Element* p_node_parent, const int offset_adjustment) override;
-    virtual bool to_sqlite(sqlite3* pDb, const gint64 node_id, const int offset_adjustment) override;
-    virtual CtAnchWidgType get_type() override { return CtAnchWidgType::Table; }
+    void apply_width_height(const int /*parentTextWidth*/) override {}
+    void to_xml(xmlpp::Element* p_node_parent, const int offset_adjustment) override;
+    bool to_sqlite(sqlite3* pDb, const gint64 node_id, const int offset_adjustment) override;
+    void set_modified_false() override;
+    CtAnchWidgType get_type() override { return CtAnchWidgType::Table; }
 
-    const CtTableMatrix& getTableMatrix() { return _tableMatrix; }
+    const CtTableMatrix& get_table_matrix() { return _tableMatrix; }
 
 protected:
     CtTableMatrix _tableMatrix;
