@@ -2,7 +2,7 @@
 #
 #       cons.py
 #
-#       Copyright 2009-2019 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2020 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -24,9 +24,14 @@ import sys
 import tempfile
 
 
-VERSION = "0.38.9"
+VERSION = "0.38.11"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
+IS_MAC_OS = False
+
+if sys.platform == 'darwin':
+    IS_MAC_OS = True
+
 if sys.platform.startswith("win"):
     IS_WIN_OS = True
     SZA_PATH = '"'+os.path.join(SHARE_PATH, "7za.exe")+'"'
@@ -153,11 +158,7 @@ NUM_CHARS_LISTNUM = len(CHARS_LISTNUM)
 CHAR_TILDE = "~"
 CHAR_MINUS = "-"
 CHAR_DQUOTE = '"'
-CHAR_SMART_DQUOTE_0 = '“'
-CHAR_SMART_DQUOTE_1 = '”'
 CHAR_SQUOTE = "'"
-CHAR_SMART_SQUOTE_0 = "‘"
-CHAR_SMART_SQUOTE_1 = "’"
 CHAR_GRAVE = "`"
 CHAR_SLASH = "/"
 CHAR_BSLASH = "\\"
@@ -179,7 +180,8 @@ CHAR_BR_CLOSE = "}"
 CHAR_CARET = "^"
 CHAR_PIPE = "|"
 CHAR_AMPERSAND = "&"
-CHARS_NOT_FOR_PASSWD = [CHAR_SQUOTE, CHAR_DQUOTE, CHAR_BSLASH, CHAR_SEMICOLON, CHAR_SPACE, CHAR_PARENTH_OPEN, CHAR_PARENTH_CLOSE, CHAR_PIPE, CHAR_AMPERSAND, CHAR_CARET]
+CHAR_DOLLAR = "$"
+CHARS_NOT_FOR_PASSWD = [CHAR_SQUOTE, CHAR_DQUOTE, CHAR_BSLASH, CHAR_SEMICOLON, CHAR_SPACE, CHAR_PARENTH_OPEN, CHAR_PARENTH_CLOSE, CHAR_PIPE, CHAR_AMPERSAND, CHAR_CARET, CHAR_DOLLAR]
 
 SPECIAL_CHAR_ARROW_RIGHT = "→"
 SPECIAL_CHAR_ARROW_RIGHT2 = "⇒"
