@@ -2,7 +2,7 @@
 #
 #       machines.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2019 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,8 +19,16 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, xml.dom.minidom, re, base64, copy, StringIO
-import cons, config, support, exports
+import gtk
+import xml.dom.minidom
+import re
+import base64
+import copy
+import StringIO
+import cons
+import config
+import support
+import exports
 
 
 def get_blob_buffer_from_pixbuf(pixbuf):
@@ -31,7 +39,7 @@ def get_blob_buffer_from_pixbuf(pixbuf):
     return blob_buffer
 
 def get_pixbuf_from_png_blob_buffer(blob_buffer):
-    """Encoded Buffer To Pixbuf"""
+    """Raw Buffer To Pixbuf"""
     pixbuf_loader = gtk.gdk.pixbuf_loader_new_with_mime_type("image/png")
     try:
         pixbuf_loader.write(blob_buffer)
@@ -40,6 +48,11 @@ def get_pixbuf_from_png_blob_buffer(blob_buffer):
     except: pixbuf = None
     return pixbuf
 
+def get_pixbuf_from_encoded_buffer(encoded_buffer):
+    """Encoded Buffer To Pixbuf"""
+    blob_buffer = base64.b64decode(encoded_buffer)
+    return get_pixbuf_from_png_blob_buffer(blob_buffer)
+
 def get_encoded_buffer_from_pixbuf(pixbuf):
     """Pixbuf To Encoded Buffer"""
     io = StringIO.StringIO()
@@ -47,16 +60,7 @@ def get_encoded_buffer_from_pixbuf(pixbuf):
     encoded_buffer = base64.b64encode(io.getvalue())
     return encoded_buffer
 
-def get_pixbuf_from_encoded_buffer(encoded_buffer):
-    """Encoded Buffer To Pixbuf"""
-    pixbuf_loader = gtk.gdk.pixbuf_loader_new_with_mime_type("image/png")
-    try:
-        pixbuf_loader.write(base64.b64decode(encoded_buffer))
-        pixbuf_loader.close()
-        pixbuf = pixbuf_loader.get_pixbuf()
-    except: pixbuf = None
-    return pixbuf
-
+# legacy
 def get_pixbuf_from_png_encoded_string(png_encoded_string):
     """PNG Encoded String To Pixbuf"""
     fd = open(cons.IMG_PATH, "wb")
@@ -108,24 +112,25 @@ class XMLHandler:
                     node_sequence += 1
                     self.append_tree_node(dom_iter, tree_father, discard_ids, node_sequence)
                 elif dom_iter.nodeName == "bookmarks":
-                    self.dad.bookmarks = dom_iter.attributes['list'].value.split(",")
-                    for node_id_str in self.dad.bookmarks:
-                        tree_iter = self.dad.get_tree_iter_from_node_id(int(node_id_str))
-                        if tree_iter:
-                            self.dad.update_node_pre_icon(tree_iter, stock_id="pin")
+                    if dom_iter.attributes['list'].value:
+                        self.dad.bookmarks = dom_iter.attributes['list'].value.split(",")
                 dom_iter = dom_iter.nextSibling
             return True
 
     def append_tree_node(self, dom_iter, tree_father, discard_ids, node_sequence):
         """Given the dom_iter node, adds it to the tree"""
-        if not discard_ids and dom_iter.hasAttribute('unique_id'):
-            unique_id = long(dom_iter.attributes['unique_id'].value)
-        else: unique_id = self.dad.node_id_get()
-        if dom_iter.hasAttribute('tags'): node_tags = dom_iter.attributes['tags'].value
-        else: node_tags = ""
+        original_id = long(dom_iter.attributes['unique_id'].value) if dom_iter.hasAttribute('unique_id') else self.dad.node_id_get()
+        if discard_ids is None:
+            unique_id = original_id
+        else:
+            unique_id = self.dad.node_id_get(original_id, discard_ids)
+        node_tags = dom_iter.attributes['tags'].value if dom_iter.hasAttribute('tags') else ""
         if node_tags: self.dad.tags_add_from_node(node_tags)
-        if dom_iter.hasAttribute('readonly'): readonly = (dom_iter.attributes['readonly'].value == "True")
-        else: readonly = False
+        readonly = (dom_iter.attributes['readonly'].value == "True") if dom_iter.hasAttribute('readonly') else False
+        custom_icon_id = int(dom_iter.attributes['custom_icon_id'].value) if dom_iter.hasAttribute('custom_icon_id') else 0
+        is_bold = (dom_iter.attributes['is_bold'].value == "True") if dom_iter.hasAttribute('is_bold') else False
+        foreground = dom_iter.attributes['foreground'].value if dom_iter.hasAttribute('foreground') else ""
+        if not foreground: foreground = None
         syntax_highlighting = dom_iter.attributes['prog_lang'].value
         if syntax_highlighting not in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]\
         and syntax_highlighting not in self.dad.available_languages:
@@ -133,14 +138,16 @@ class XMLHandler:
             if syntax_highlighting not in self.dad.available_languages:
                 syntax_highlighting = cons.RICH_TEXT_ID
         node_depth = 0 if not tree_father else self.dad.treestore.iter_depth(tree_father)+1
-        cherry = self.dad.get_node_icon(node_depth, syntax_highlighting)
+        cherry = self.dad.get_node_icon(node_depth, syntax_highlighting, custom_icon_id)
+        ts_creation = float(dom_iter.attributes['ts_creation'].value) if dom_iter.hasAttribute('ts_creation') else 0
+        ts_lastsave = float(dom_iter.attributes['ts_lastsave'].value) if dom_iter.hasAttribute('ts_lastsave') else 0
         curr_buffer = self.dad.buffer_create(syntax_highlighting)
         if syntax_highlighting != cons.RICH_TEXT_ID: curr_buffer.begin_not_undoable_action()
         # loop into rich text, write into the buffer
         child_dom_iter = dom_iter.firstChild
         while child_dom_iter != None:
             if child_dom_iter.nodeName == "rich_text":
-                self.rich_text_deserialize(curr_buffer, child_dom_iter)
+                self.rich_text_deserialize(curr_buffer, child_dom_iter, discard_ids)
             elif child_dom_iter.nodeName == "encoded_png": self.image_deserialize(curr_buffer, child_dom_iter, 2)
             elif child_dom_iter.nodeName == "table": self.table_deserialize(curr_buffer, child_dom_iter)
             elif child_dom_iter.nodeName == "codebox": self.codebox_deserialize(curr_buffer, child_dom_iter)
@@ -159,9 +166,15 @@ class XMLHandler:
                                                             node_sequence,
                                                             node_tags,
                                                             readonly,
-                                                            None])
+                                                            None,
+                                                            custom_icon_id,
+                                                            support.get_pango_weight(is_bold),
+                                                            foreground,
+                                                            ts_creation,
+                                                            ts_lastsave])
+        self.dad.update_node_aux_icon(tree_iter)
         self.dad.nodes_names_dict[unique_id] = self.dad.treestore[tree_iter][1]
-        if discard_ids:
+        if discard_ids is not None:
             # we are importing nodes
             self.dad.ctdb_handler.pending_new_db_node(unique_id)
         # loop for child nodes
@@ -243,18 +256,26 @@ class XMLHandler:
                 image_justification=justification,
                 text_buffer=curr_buffer)
 
-    def rich_text_deserialize(self, curr_buffer, dom_node):
+    def rich_text_deserialize(self, curr_buffer, dom_node, discard_ids=None):
         """From the XML rich text to the SourceBuffer"""
-        if dom_node.firstChild: text = dom_node.firstChild.data
-        else: text = ""
-        tag_names = []
-        for tag_property in cons.TAG_PROPERTIES:
-            if dom_node.hasAttribute(tag_property):
-                property_value = dom_node.attributes[tag_property].value
-                if property_value: tag_names.append(self.dad.apply_tag_exist_or_create(tag_property, property_value))
-        tags_num = len(tag_names)
-        if tags_num == 0: curr_buffer.insert(curr_buffer.get_end_iter(), text)
-        else: curr_buffer.insert_with_tags_by_name(curr_buffer.get_end_iter(), text, *tag_names)
+        if dom_node.firstChild:
+            text = dom_node.firstChild.data
+            tag_names = []
+            for tag_property in cons.TAG_PROPERTIES:
+                if dom_node.hasAttribute(tag_property):
+                    property_value = dom_node.attributes[tag_property].value
+                    if property_value:
+                        if discard_ids is not None and property_value.startswith(cons.LINK_TYPE_NODE):
+                            property_value_vec = property_value.split()
+                            original_id = int(property_value_vec[1])
+                            imported_id = self.dad.node_id_get(original_id, discard_ids)
+                            property_value_vec[1] = str(imported_id)
+                            property_value = cons.CHAR_SPACE.join(property_value_vec)
+                            #print original_id, imported_id
+                        tag_names.append(self.dad.apply_tag_exist_or_create(tag_property, property_value))
+            tags_num = len(tag_names)
+            if tags_num == 0: curr_buffer.insert(curr_buffer.get_end_iter(), text)
+            else: curr_buffer.insert_with_tags_by_name(curr_buffer.get_end_iter(), text, *tag_names)
 
     def treestore_node_to_dom(self, node_iter):
         """Given a treestore iter returns the CherryTree rich text"""
@@ -264,12 +285,12 @@ class XMLHandler:
         return self.dom.toxml()
 
     def treestore_sel_node_only_to_dom(self, tree_iter, sel_range=None):
-        """Parse the Given Node and Subnodes and Generate an XML Cherry Tree Document"""
+        """Parse the Given Node and Generate an XML Cherry Tree Document"""
         if "dom" in dir(self): del self.dom
         self.dom = xml.dom.minidom.Document()
         cherrytree = self.dom.createElement(cons.APP_NAME)
         self.dom.appendChild(cherrytree)
-        # given node and subnodes parsing
+        # given node parsing
         self.append_dom_node(tree_iter, cherrytree, to_disk=True, skip_children=True, sel_range=sel_range)
         return self.dom.toxml()
 
@@ -289,13 +310,14 @@ class XMLHandler:
         self.dom = xml.dom.minidom.Document()
         cherrytree = self.dom.createElement(cons.APP_NAME)
         self.dom.appendChild(cherrytree)
+        # bookmarks first
+        self.append_bookmarks(cherrytree)
         # full tree parsing
         tree_iter = self.dad.treestore.get_iter_first()
         while tree_iter != None:
             self.append_dom_node(tree_iter, cherrytree, to_disk=True)
             tree_iter = self.dad.treestore.iter_next(tree_iter)
-        self.append_bookmarks(cherrytree)
-        return self.dom.toxml()
+        return self.dom.toprettyxml()
 
     def append_bookmarks(self, dom_father):
         """Adds the bookmarks data to the DOM"""
@@ -313,6 +335,13 @@ class XMLHandler:
         dom_iter.setAttribute("prog_lang", programming_language)
         dom_iter.setAttribute("tags", self.dad.treestore[tree_iter][6])
         dom_iter.setAttribute("readonly", str(self.dad.treestore[tree_iter][7]))
+        dom_iter.setAttribute("custom_icon_id", str(self.dad.treestore[tree_iter][9]))
+        dom_iter.setAttribute("is_bold", str(support.get_pango_is_bold(self.dad.treestore[tree_iter][10])))
+        dom_iter.setAttribute("ts_creation", str(self.dad.treestore[tree_iter][12]))
+        dom_iter.setAttribute("ts_lastsave", str(self.dad.treestore[tree_iter][13]))
+        foreground = self.dad.treestore[tree_iter][11]
+        if not foreground: foreground = ""
+        dom_iter.setAttribute("foreground", foreground)
         dom_father.appendChild(dom_iter)
         # allocate and init the rich text attributes
         self.curr_attributes = {}
@@ -518,7 +547,7 @@ class XMLHandler:
                     curr_node_level = self.dad.treestore.iter_depth(node_tree_iter)
                     node_name = self.dad.treestore[node_tree_iter][1].replace(cons.CHAR_NEWLINE, cons.CHAR_SPACE).replace(cons.CHAR_CR, "")
                     tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value)]
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*curr_node_level*cons.CHAR_SPACE + cons.CHAR_LISTARR + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*curr_node_level*cons.CHAR_SPACE + self.dad.chars_toc[0] + cons.CHAR_SPACE)
                     curr_offset += 2 + 2*curr_node_level
                     text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                     text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), node_name, *tag_names)
@@ -529,13 +558,13 @@ class XMLHandler:
                 tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0])]
                 if not element[0]: continue
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[1] + cons.CHAR_SPACE)
                     curr_offset += 4 + 2*curr_node_level
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[2] + cons.CHAR_SPACE)
                     curr_offset += 6 + 2*curr_node_level
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[3] + cons.CHAR_SPACE)
                     curr_offset += 8 + 2*curr_node_level
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -545,7 +574,7 @@ class XMLHandler:
                 curr_offset += 1
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
         return toc_list_per_node
-        
+
     def toc_insert_all_iter(self, top_tree_iter):
         """Iterate on nodes for toc_insert_all"""
         self.dad.treeview_safe_set_cursor(top_tree_iter)
@@ -618,13 +647,13 @@ class XMLHandler:
                 tag_names = []
                 tag_names.append(self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0]))
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), self.dad.chars_toc[1] + cons.CHAR_SPACE)
                     curr_offset += 2
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + self.dad.chars_toc[2] + cons.CHAR_SPACE)
                     curr_offset += 4
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + self.dad.chars_toc[3] + cons.CHAR_SPACE)
                     curr_offset += 6
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -669,6 +698,7 @@ class StateMachine:
 
     def __init__(self, dad):
         """Machine boot"""
+        self.dad = dad
         self.nodes_vectors = {}
         self.nodes_indexes = {}
         self.nodes_indicators = {}
@@ -677,7 +707,7 @@ class StateMachine:
         # indicator 0 -> current text in node_vector matches the textbuffer
         # indicator 1 -> textbuffer is ahead, but only of non alphanumeric chars
         # indicator 2 -> textbuffer is ahead with alphanumeric chars
-        self.dad = dad
+        self.not_undoable_timeslot_set(False)
 
     def get_embedded_pixbufs_tables_codeboxes(self, text_buffer, for_print=0, sel_range=None):
         """Retrieve the list of Images Embedded into the Buffer"""
@@ -707,8 +737,8 @@ class StateMachine:
 
     def table_to_dict(self, anchor):
         """Given an Anchor, Returns the Embedded Table as a dictionary"""
-        columns_num = len(anchor.headers)
         table_dict = {'matrix':[], 'col_min': anchor.table_col_min, 'col_max': anchor.table_col_max}
+        columns_num = len(anchor.headers)
         tree_iter = anchor.liststore.get_iter_first()
         while tree_iter != None:
             row = []
@@ -740,12 +770,12 @@ class StateMachine:
     def get_iter_alignment(self, iter_text):
         """Get the Alignment Value of the given Iter"""
         align_center = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_CENTER)
-        align_fill = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_FILL)
-        align_right = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_RIGHT)
         if iter_text.has_tag(self.dad.tag_table.lookup(align_center)):
             return cons.TAG_PROP_CENTER
+        align_fill = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_FILL)
         if iter_text.has_tag(self.dad.tag_table.lookup(align_fill)):
             return cons.TAG_PROP_FILL
+        align_right = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_RIGHT)
         if iter_text.has_tag(self.dad.tag_table.lookup(align_right)):
             return cons.TAG_PROP_RIGHT
         return cons.TAG_PROP_LEFT
@@ -790,18 +820,13 @@ class StateMachine:
 
     def reset(self):
         """State Machine Reset"""
-        del self.nodes_vectors
-        del self.nodes_indexes
-        del self.nodes_indicators
-        del self.visited_nodes_list
-        del self.visited_nodes_idx
         self.nodes_vectors = {}
         self.nodes_indexes = {}
         self.nodes_indicators = {}
         self.visited_nodes_list = []
         self.visited_nodes_idx = 0
 
-    def requested_previous_visited(self):
+    def requested_visited_previous(self):
         """Requested the Previous Visited Node"""
         if self.visited_nodes_idx != None and self.visited_nodes_idx > 0:
             self.visited_nodes_idx -= 1
@@ -810,7 +835,7 @@ class StateMachine:
             #print "self.visited_nodes_idx", self.visited_nodes_idx
             return None
 
-    def requested_next_visited(self):
+    def requested_visited_next(self):
         """Requested the Next Visited Node"""
         if self.visited_nodes_idx != None and self.visited_nodes_idx < len(self.visited_nodes_list) - 1:
             self.visited_nodes_idx += 1
@@ -820,16 +845,13 @@ class StateMachine:
             #print "last_index",  len(self.visited_nodes_list) - 1
             return None
 
-    def forget_last_visited(self):
-        """Remove the latest element from the list of the visited nodes"""
-        self.visited_nodes_idx -= 1
-        self.visited_nodes_list.pop()
-
     def node_selected_changed(self, node_id):
         """When a New Node is Selected"""
-        if not self.dad.go_bk_fw_click:
+        if self.dad.user_active and not self.dad.go_bk_fw_click:
             last_index = len(self.visited_nodes_list) - 1
             if self.visited_nodes_idx != last_index: del self.visited_nodes_list[self.visited_nodes_idx+1:last_index+1]
+            if node_id in self.visited_nodes_list:
+                self.visited_nodes_list.remove(node_id)
             self.visited_nodes_list.append(node_id)
             self.visited_nodes_idx = len(self.visited_nodes_list) - 1
         if node_id not in self.nodes_vectors:
@@ -852,7 +874,7 @@ class StateMachine:
         elif alphanum == None: # self.nodes_indicators[node_id] == 2 and non alphanumeric transition
             self.update_state()
 
-    def requested_previous_state(self, node_id):
+    def requested_state_previous(self, node_id):
         """A Previous State, if Existing, is Requested"""
         if self.curr_index_is_last_index(node_id):
             self.update_state()
@@ -860,11 +882,11 @@ class StateMachine:
             self.nodes_indexes[node_id] -= 1
         return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
-    def requested_current_state(self, node_id):
+    def requested_state_current(self, node_id):
         """The current state is requested"""
         return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
-    def requested_subsequent_state(self, node_id):
+    def requested_state_subsequent(self, node_id):
         """A Subsequent State, if Existing, is Requested"""
         if self.nodes_indexes[node_id] < (len(self.nodes_vectors[node_id])-1):
             self.nodes_indexes[node_id] += 1
@@ -872,9 +894,9 @@ class StateMachine:
 
     def delete_states(self, node_id):
         """Delete the states for the given node_id"""
-        del self.nodes_indexes[node_id]
-        del self.nodes_vectors[node_id]
-        del self.nodes_indicators[node_id]
+        if node_id in self.nodes_indexes: del self.nodes_indexes[node_id]
+        if node_id in self.nodes_vectors: del self.nodes_vectors[node_id]
+        if node_id in self.nodes_indicators: del self.nodes_indicators[node_id]
         if node_id in self.visited_nodes_list:
             self.visited_nodes_list.remove(node_id)
             self.visited_nodes_idx = len(self.visited_nodes_list)-1
@@ -885,9 +907,15 @@ class StateMachine:
         last_index = len(self.nodes_vectors[node_id]) - 1
         return curr_index == last_index
 
+    def not_undoable_timeslot_set(self, not_undoable_val):
+        self._not_undoable_timeslot = not_undoable_val
+
+    def not_undoable_timeslot_get(self):
+        return self._not_undoable_timeslot
+
     def update_state(self):
         """Update the state for the given node_id"""
-        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
+        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID or self.not_undoable_timeslot_get():
             return
         node_id = self.dad.treestore[self.dad.curr_tree_iter][3]
         if not self.curr_index_is_last_index(node_id):
